"""Run this via `make climate_categories/data/gas.yaml` in the main directory."""

import pathlib

import openscm_units
import openscm_units.data
import openscm_units.data.mixtures

import climate_categories

OUTPATH = pathlib.Path("./climate_categories/data/gas.yaml")
OUTPATH_PICKLE = pathlib.Path("./climate_categories/data/gas.pickle")


def main():
    categories = openscm_standard_gases()
    categories.update(openscm_mixtures())

    # specific fixes
    categories["OC"]["title"] = "organic carbon"
    categories["OC"]["comment"] = "organic carbon"

    categories["VOC"]["title"] = "volatile organic compounds"
    categories["VOC"]["comment"] = "non-methane volatile organic compounds"

    spec = {
        "name": "gas",
        "title": "climate-forcing gases",
        "comment": "Gases and other climate-forcing substances",
        "references": "Derived from openscm_units "
        "(https://github.com/openscm/openscm-units) 'standard gases' and mixtures.",
        "last_update": "2021-05-27",
        "version": "0.3.0",
        "institution": "openscm",
        "categories": categories,
        "hierarchical": True,
        "total_sum": False,
    }

    gas = climate_categories.HierarchicalCategorization.from_spec(spec)
    gas.to_yaml(OUTPATH)
    gas.to_pickle(OUTPATH_PICKLE)


def openscm_mixtures():
    categories = {}
    m = openscm_units.data.mixtures.MIXTURES

    for code, constituents in m.items():
        cstr = ", ".join(
            (f"{constituents[const][0]}% {const}" for const in constituents)
        )
        categories[code] = {
            "title": code,
            "comment": f"The refrigerant {code}, which is a mixture " f"of {cstr}.",
        }

    categories["mixtures"] = {
        "title": "refrigerant mixtures",
        "children": [list(categories.keys())],
    }

    return categories


def openscm_standard_gases():
    categories = {}
    # This is not public API, will have to fix it when openscm_units changes
    sg = openscm_units._unit_registry._STANDARD_GASES

    for (oscm_gas_code, oscm_gas_spec) in sg.items():
        if isinstance(oscm_gas_spec, str):  # base entity
            code = oscm_gas_code
            title = oscm_gas_spec.replace("_", " ")
            categories[code] = {
                "title": title,
            }
            if code != oscm_gas_spec and " " not in oscm_gas_spec:
                categories[code]["alternative_codes"] = [oscm_gas_spec]
        else:  # derived entity or alias
            definition = oscm_gas_spec[0]
            oscm_altcodes = oscm_gas_spec[1:]
            if definition in categories:  # alias
                if "alternative_codes" not in categories[definition]:
                    categories[definition]["alternative_codes"] = []
                categories[definition]["alternative_codes"].append(oscm_gas_code)
                categories[definition]["alternative_codes"] += oscm_altcodes
            else:  # derived entity
                code = oscm_gas_code
                if len(oscm_altcodes) > 1:
                    raise ValueError(f"Unexpected entry for {code}")
                title = oscm_altcodes[0].replace("_", " ")
                categories[code] = {
                    "title": title,
                }
                if code != oscm_altcodes[0]:
                    categories[code]["alternative_codes"] = oscm_altcodes

    return categories


if __name__ == "__main__":
    main()



from typing import Iterable, Set

from . import _categories


def search_code(
    code: str, cats: Iterable[_categories.Categorization]
) -> Set[_categories.Category]:
    """Search for the given code in the given categorizations."""
    res = set()
    for cat in cats:
        if code in cat.all_keys():
            res.add(cat[code])
    return res



"""Access to all categorizations is provided directly at the module level, using the
names of categorizations. To access the example categorization `Excat`, simply use
`climate_categories.Excat` .
"""

__author__ = """Mika Pflüger"""
__email__ = "mika.pflueger@pik-potsdam.de"
__version__ = "0.6.1"

import importlib
import importlib.resources
import typing

from . import _categories, data, search
from ._categories import Categorization  # noqa: F401
from ._categories import from_spec  # noqa: F401
from ._categories import from_yaml  # noqa: F401
from ._categories import HierarchicalCategorization, from_pickle

cats = {}


def _read_pickle_hier(name) -> HierarchicalCategorization:
    with importlib.resources.open_binary(data, f"{name}.pickle") as fd:
        _cat = from_pickle(fd)
    _cat._cats = cats
    cats[_cat.name] = _cat
    return _cat


# not used at the moment, uncomment if needed for non-hierarchical Categorizations
# def _read_pickle_nh(name) -> Categorization:
#    with importlib.resources.open_binary(data, f"{name}.pickle") as fd:
#        _cat = from_pickle(fd)
#    _cat._cats = cats
#    cats[_cat.name] = _cat
#    return _cat


# do this explicitly to help static analysis tools
IPCC1996 = _read_pickle_hier("IPCC1996")
IPCC2006 = _read_pickle_hier("IPCC2006")
IPCC2006_PRIMAP = _read_pickle_hier("IPCC2006_PRIMAP")
CRF1999 = _read_pickle_hier("CRF1999")
CRFDI = _read_pickle_hier("CRFDI")
CRFDI_class = _read_pickle_hier("CRFDI_class")
BURDI = _read_pickle_hier("BURDI")
BURDI_class = _read_pickle_hier("BURDI_class")
GCB = _read_pickle_hier("GCB")
RCMIP = _read_pickle_hier("RCMIP")
gas = _read_pickle_hier("gas")


def find_code(code: str) -> typing.Set[_categories.Category]:
    """Search for the given code in all included categorizations."""
    return search.search_code(code, cats.values())


__all__ = [
    "cats",
    "Categorization",
    "HierarchicalCategorization",
    "find_code",
    "from_pickle",
    "from_spec",
    "from_yaml",
] + list(cats.keys())



"""Run this via `make climate_categories/data/BURDI.yaml` in the main directory."""
import datetime
import pathlib

import natsort
import treelib
from unfccc_di_api import UNFCCCApiReader

import climate_categories

OUTPATH = pathlib.Path("./climate_categories/data/BURDI.yaml")

# there are some categories with different IDs, but the same code and title.
# These are the categories directly under the top-level totals (e.g. 1. Energy),
# which are included in the category hierarchy once under the Total with and once
# under the Total without LULUCF.
#
# they can be safely merged because they are not actually different.


def parse_categories(r):
    categories = {}
    # there are unused intermediate categories needed for tree traversal only existing
    # in the tree and also categories without metadata only in the variables
    all_category_ids = set(r.non_annex_one_reader.category_tree.nodes.keys()).union(
        set(r.non_annex_one_reader.variables["categoryId"])
    )
    for category_id in all_category_ids:
        if category_id == 15164:
            # generic "Totals" category that is violating the total_sum rule and unused
            continue

        try:
            raw_category = r.non_annex_one_reader.category_tree.nodes[category_id].tag
        except KeyError:
            if category_id == 10502:
                raw_category = "Total land area"
            elif category_id == 10503:
                raw_category = "GDP"
            elif category_id == 10504:
                raw_category = "Total population"
            else:
                raw_category = f"{category_id} Unknown category no. {category_id}"

        if raw_category[0].isnumeric():
            code, title = raw_category.split(maxsplit=1)
            if code.endswith("."):
                altcodes = [code, str(category_id)]
                code = code[:-1]
            else:
                altcodes = [str(category_id)]
        elif raw_category == "Memo Items":
            # needs special handling because it doesn't have a proper code but
            # is included twice with different category_ids
            code = "14568"
            title = raw_category
            altcodes = ["24568"]
        else:
            code = str(category_id)
            title = raw_category
            altcodes = []

        if code not in categories:
            categories[code] = {"title": title}

        altcode = code.replace(".", "")
        if altcode != code:
            altcodes.append(altcode)
            altcodes.append(code.replace(".", " "))

        if altcodes:
            if "alternative_codes" not in categories[code]:
                categories[code]["alternative_codes"] = []
            for ac in altcodes:
                if ac not in categories[code]["alternative_codes"]:
                    categories[code]["alternative_codes"].append(ac)

        if "info" not in categories[code]:
            categories[code]["info"] = {}
        if "numerical_ids" not in categories[code]["info"]:
            categories[code]["info"]["numerical_ids"] = []
        categories[code]["info"]["numerical_ids"].append(str(category_id))

    return categories


def add_relationships(r, categories):
    for code in categories:
        child_codes = []
        for nid in categories[code]["info"]["numerical_ids"]:
            try:
                for child in r.non_annex_one_reader.category_tree.children(int(nid)):
                    child_code = [
                        xcode
                        for xcode, x in categories.items()
                        if str(int(child.identifier)) in x["info"]["numerical_ids"]
                    ][0]
                    child_codes.append(child_code)
            except treelib.tree.NodeIDAbsentError:
                # parent code is not included in the metadata
                # can't guess children without metadata
                pass

        if child_codes:
            categories[code]["children"] = [child_codes]


def sort_categories(categories):
    sorted_categories = {}
    # start with "special" categories without a normal X.Y etc. numbering
    special = []
    normal = []
    for code in categories:
        if code.isnumeric() and int(code) > 10:
            special.append(code)
        else:
            normal.append(code)

    sorted_codes = sorted(special, key=int) + natsort.natsorted(normal)

    for cat in sorted_codes:
        sorted_categories[cat] = categories[cat]

    return sorted_categories


def main():
    r = UNFCCCApiReader()
    categories = parse_categories(r)
    add_relationships(r, categories)
    categories = sort_categories(categories)

    spec = {
        "name": "BURDI",
        "title": "BUR GHG emission categories (DI query interface)",
        "comment": "Biannual Update Report categories of GHG emissions and removals "
        "and some other quantities as obtained from the di.unfccc.int flexible query "
        "interface",
        "references": "https://unfccc.int/process-and-meetings/"
        "transparency-and-reporting/greenhouse-gas-data/data-interface-help#eq-7 and "
        "decision 17/CP.8, 'Guidelines for the preparation of national communications "
        "from Parties not included in Annex I to the Convention' available at "
        "https://unfccc.int/files/meetings/workshops/other_meetings/application/pdf/"
        "dec17-cp.pdf",
        "institution": "UNFCCC",
        "last_update": "2021-11-05",
        "hierarchical": True,
        "total_sum": True,
        "categories": categories,
        "canonical_top_level_category": "24540",  # Total including LULUCF
    }

    CRFDI = climate_categories.HierarchicalCategorization.from_spec(spec)

    CRFDI.to_yaml(OUTPATH)

    climate_categories.HierarchicalCategorization.from_yaml(OUTPATH)


if __name__ == "__main__":
    main()



import requests

resp = requests.get("https://zenodo.org/api/records/4590232").json()

new_link = resp["links"]["doi"]
new_doi = resp["metadata"]["doi"]
new_date = resp["metadata"]["publication_date"]
new_title = resp["metadata"]["title"]

citation = f"""Citation
--------
If you use this library and want to cite it, please cite it as:

Mika Pflüger, Annika Günther, Johannes Gütschow, and Robert Gieseke. ({new_date}).
{new_title}.
Zenodo. {new_link}
"""

with open("README.rst", "r") as fd:
    old_content = fd.read().splitlines(keepends=True)

with open("README.rst", "w") as fd:
    skip_to_next_section = False
    i = 0
    while True:
        try:
            line = old_content[i]
        except IndexError:
            break
        if line == "Citation\n":
            fd.write(citation)
            skip_to_next_section = True
            i += 2
        elif skip_to_next_section:
            if line.startswith("---"):
                fd.write("\n")
                fd.write(old_content[i - 1])
                fd.write(line)
                skip_to_next_section = False
            i += 1
        else:
            fd.write(line)
            i += 1

    fd.truncate()



"""Convert a categorization stored in a StrictYaml file to a python pickle for faster
loading."""

import sys

import climate_categories


def main():
    cat = climate_categories.from_yaml(sys.argv[1])
    cat.to_pickle(sys.argv[2])


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import datetime
import pathlib
import sys

version = sys.argv[1]

with open("CHANGELOG.rst") as fd:
    old_content = fd.read().splitlines(keepends=True)

with open("CHANGELOG.rst", "w") as fd, open(
    ".changelog_latest_version.rst", "w"
) as lfd:
    # Write header
    fd.writelines(old_content[:4])

    # New version
    headline = f"{version} ({datetime.date.today().isoformat()})"
    new_version = f"{headline}\n" + "-" * len(headline) + "\n"
    ch_dir = pathlib.Path("changelog_unreleased")
    for changelog_file in ch_dir.iterdir():
        new_version += changelog_file.read_text()
        changelog_file.unlink()

    fd.write(new_version)
    lfd.write(new_version)

    fd.write("\n")

    # Rest
    fd.writelines(old_content[4:])



import climate_categories


def HierEx():
    spec = {
        "name": "HierEx",
        "title": "Hierarchical Example Categorization",
        "comment": "A simple hierarchical categorization",
        "references": "mailto:mika.pflueger@pik-potsdam.de",
        "institution": "PIK",
        "last_update": "2021-10-12",
        "hierarchical": True,
        "version": "1",
        "total_sum": "True",
        "canonical_top_level_category": "0",
        "categories": {
            "0": {"title": "Total", "children": [["1", "2", "3"]]},
            "1": {"title": "Sector 1"},
            "2": {"title": "Sector 2"},
            "3": {"title": "Sector 3"},
        },
    }
    return climate_categories.HierarchicalCategorization.from_spec(spec)


def HierAltEx():
    spec = {
        "name": "HierAltEx",
        "title": "Hierarchical Example Categorization with Alternatives",
        "comment": "A simple hierarchical categorization with alternative child sets",
        "references": "mailto:mika.pflueger@pik-potsdam.de",
        "institution": "PIK",
        "last_update": "2021-10-12",
        "hierarchical": True,
        "version": "1",
        "total_sum": "True",
        "canonical_top_level_category": "0",
        "categories": {
            "0": {"title": "Total", "children": [["1", "2", "3"], ["a", "b", "c"]]},
            "1": {"title": "Sector 1"},
            "2": {"title": "Sector 2"},
            "3": {"title": "Sector 3"},
            "a": {"title": "Fuel a"},
            "b": {"title": "Fuel b"},
            "c": {"title": "Fuel c"},
        },
    }
    return climate_categories.HierarchicalCategorization.from_spec(spec)
